@@ -33,8 +33,8 @@ fishkent <- function(x, B = 999) {
     for (i in 1:B) {
       nu <- sample(1:n, n, replace = TRUE)
       z <- x[nu, ]
-      estim <- vmf(z)
-      k <- estim$k  ## the estimated concentration parameter
+      estim <- vmf(z, fast = TRUE)
+      k <- estim$kappa  ## the estimated concentration parameter
       ## under the H0, that the Fisher distribution is true
       mu <- estim$mu  ## the estimated mean direction under H0
       P <- i3 -  tcrossprod(e1 - mu) / (1 - mu[1])
